class ExecExp:
    """
    Exec expression wrapper
    :param exec: Pass this argument to enable
    :param exec_crypt: 'escape' for escape, 'base64' for base64, 'base64-lzma' for base64+lzma
    """

    priority = -2

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        if "exec" not in self.options:
            return code
        if "exec_crypt" not in self.options:
            return "exec(" + repr(code) + ")"
        elif self.options["exec_crypt"] == "base64":
            import base64

            return (
                'exec(__import__("base64").b64decode('
                + repr(base64.b64encode(code.encode()))
                + ").decode())"
            )
        elif self.options["exec_crypt"] == "base64-lzma":
            import lzma, base64

            return (
                'exec(__import__("lzma").decompress(__import__("base64").b64decode('
                + repr(base64.b64encode(lzma.compress(code.encode())))
                + ")).decode())"
            )
        elif self.options["exec_crypt"] == "escape":
            return 'exec("' + "".join("\\u{:04x}".format(ord(i)) for i in code) + '")'


__all__ = ["ExecExp"]



import warnings


def getint1(x):
    one = "__name__.__len__().__bool__().__int__()"
    one += (".__add__(" + one + ")") * (x - 1)
    return one


def getint2(x):
    one = "__name__.__len__().__bool__().__int__()"
    two = one + ".__add__(" + one + ")"
    if x == 1:
        return one
    if x == 2:
        return two
    return (
        getint2(x >> 1) + ".__mul__(" + two + ")" + (".__add__(" + one + ")") * (x & 1)
    )


def getint(x):
    g1, g2 = getint1(x), getint2(x)
    if len(g1) < len(g2):
        return g1
    return g2


def getchar(x):
    two = getint(2)
    c = "__name__.__class__.__call__.__name__.__getitem__(" + two + ")"
    h = "__name__.__hash__.__name__.__getitem__(" + two + ")"
    r = "__name__.__rmul__.__name__.__getitem__(" + two + ")"
    cchr = (
        "__builtins__.__getattribute__(" + c + ".__add__(" + h + ").__add__(" + r + "))"
    )
    return cchr + ".__call__(" + x + ")"


def geteval(x):
    one, three, four = getint(1), getint(3), getint(4)
    e = "__name__.__class__.__class__.__name__.__getitem__(" + three + ")"
    v = (
        "__name__.__class__.__call__().__len__().__invert__.__name__.__getitem__("
        + four
        + ")"
    )
    a = (
        "__name__.__class__.__call__().__len__().__truediv__("
        + one
        + ").__class__.__name__.__getitem__("
        + three
        + ")"
    )
    l = (
        "__name__.__class__.__call__().__len__().__truediv__("
        + one
        + ").__class__.__name__.__getitem__("
        + one
        + ")"
    )
    eeval = (
        "__builtins__.__getattribute__("
        + e
        + ".__add__("
        + v
        + ").__add__("
        + a
        + ").__add__("
        + l
        + "))"
    )
    return eeval + ".__call__(" + x + ")"


def magicify(code):
    if not code:
        return ""
    code = "exec(" + repr(code) + ")"
    res = getchar(getint(ord(code[0])))
    for i, j in enumerate(code):
        if i == 0:
            continue
        res = res + ".__add__(" + getchar(getint(ord(j))) + ")"
    return geteval(res)


class Magic:
    """
    Converts normal Python code into code that only uses magic functions (inspired by this
    article: https://esolangs.org/wiki/Python_is_Magic).
    This is slow when processing large programs, and will largely increase the program size, so it is only supposed to
    be used for small programs.
    If the generated program is larger than 10KiB, you are warned.
    """

    priority = -1

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        if "magic" not in self.options:
            return code
        result = magicify(code)
        if len(result) > 10240:
            warnings.warn("The converted program is larger than 10KiB")
        return result


__all__ = ["Magic"]



from random import *
import pygments
from pygments.lexers import PythonLexer
from pygments.token import *

hex_cache = set()


def hex_renamer():
    r = "_" + "0x%x" % (randint(0x100000, 0xFFFFFF))
    while r in hex_cache:
        r = "_" + "0x%x" % (randint(0x100000, 0xFFFFFF))
    return r


underscore_cache = (
    1  # Avoid using one underline as variable names because it is usable in Python
)


def underscore_renamer():
    global underscore_cache
    underscore_cache += 1
    return "_" * underscore_cache


char_cache = set()


def character_renamer():
    r = "".join(
        choice("qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM")
        for i in range(20)
    )
    while r in char_cache:
        r = "".join(
            choice("qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM")
            for i in range(20)
        )
    return r


ozero_cache = set()


def ozero_renamer():
    r = "O" + "".join(choice("0O") for i in range(19))
    while r in ozero_cache:
        r = "O" + "".join(choice("0O") for i in range(19))
    return r


lone_cache = set()


def lone_renamer():
    r = "l" + "".join(choice("1l") for i in range(19))
    while r in lone_cache:
        r = "l" + "".join(choice("1l") for i in range(19))
    return r


def getzero():
    op = randint(0, 4)
    if op == 0:
        t = randint(0, 0xFFFFFFFFFFFFFFFF)
        return "{}+(-{})".format(t, t)
    if op == 1:
        t = randint(0, 0xFFFFFFFFFFFFFFFF)
        return "{}-{}".format(t, t)
    if op == 2:
        return "0*{}".format(randint(0, 0xFFFFFFFFFFFFFFFF))
    return "0//{}".format(randint(1, 0xFFFFFFFFFFFFFFFF))


def rndcode(depth, renamer):
    z = []
    if depth == 9:
        codetype = 0
    else:
        codetype = randint(int(depth < 3), 3)
    for i in range(randint(1, 10)):
        if randint(0, 1):
            z.append(("pass", depth))
        else:
            z.append(("var", renamer(), randint(0, 0xFFFFFFFFFFFFFFFF), depth))
    if depth >= 10:
        return z + []
    if codetype == 0:
        return z + []
    if codetype == 1:
        y = [("if", getzero(), depth)]
        y += rndcode(depth + 1, renamer)
        return z + y
    y = [("while", getzero(), depth)]
    y += rndcode(depth + 1, renamer)
    return z + y


def compose_code(c):
    res = ""
    for i in c:
        res += "    " * i[-1]
        if i[0] == "var":
            res += "{}={}".format(i[1], i[2])
        elif i[0] == "pass":
            res += "pass"
        elif i[0] == "if":
            res += "if {}:".format(i[1])
        else:
            res += "while {}:".format(i[1])
        res += "\n"
    return res.strip()


class AddCode:
    """
    Insert random code that does nothing
    :param add_code: Pass this argument to enable
    :param renamer: The renamer to use, 'hex' for hex renamer, 'char' for random letter renamer, 'o0' for O and 0
    renamer, 'underscore' for underscore renamer and 'l1' for l and 1 renamer, hex renamer is used if not passed.
    :param code_amount: Amount of code to insert, default is 10
    """

    priority = 2

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        global underscore_cache, hex_cache, char_cache, ozero_cache, lone_cache
        underscore_cache, ozero_cache, lone_cache, char_cache, hex_cache = (
            1,
            set(),
            set(),
            set(),
            set(),
        )
        if "add_code" not in self.options:
            return code
        result = ""
        open_bracket = 0
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if is_token_subtype(toktype, Token.String) and tokval == "\n":
                result += "\\n"
                continue
            else:
                open_bracket += (
                    tokval.count("(")
                    + tokval.count("[")
                    + tokval.count("{")
                    - tokval.count("}")
                    - tokval.count("]")
                    - tokval.count(")")
                )
                if tokval == "\n" and open_bracket:
                    continue
            result += tokval
        code = result
        if "renamer" not in self.options:
            renamer = hex_renamer
        else:
            renamer = {
                "hex": hex_renamer,
                "char": character_renamer,
                "o0": ozero_renamer,
                "underscore": underscore_renamer,
                "l1": lone_renamer,
            }[self.options["renamer"]]
        breakdown = code.split("\n")
        if "code_amount" in self.options:
            num = self.options["code_amount"]
        else:
            num = 10
        breakdown = [""] + breakdown
        for i in range(num):
            pos = randint(0, len(breakdown) - 1)
            indent = ""
            if pos == len(breakdown) - 1:
                indent = ""
            else:
                c = breakdown[pos + 1]
                for j in c:
                    if j.isspace():
                        indent += j
                    else:
                        break
            random_code = compose_code(rndcode(0, renamer))
            for j in reversed(random_code.split("\n")):
                breakdown.insert(pos + 1, indent + j)
        return "\n".join(breakdown).strip()


__all__ = ["AddCode"]



import pygments
from pygments.lexers import PythonLexer
from random import randint, choice
from pygments.token import *

hex_cache = set()


def hex_renamer():
    r = "_" + "0x%x" % (randint(0x100000, 0xFFFFFF))
    while r in hex_cache:
        r = "_" + "0x%x" % (randint(0x100000, 0xFFFFFF))
    return r


underscore_cache = (
    1  # Avoid using one underline as variable names because it is usable in Python
)


def underscore_renamer():
    global underscore_cache
    underscore_cache += 1
    return "_" * underscore_cache


char_cache = set()


def character_renamer():
    r = "".join(
        choice("qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM")
        for i in range(20)
    )
    while r in char_cache:
        r = "".join(
            choice("qwertyuiopasdfghjklzxcvbnmQWERTYUIOPASDFGHJKLZXCVBNM")
            for i in range(20)
        )
    return r


ozero_cache = set()


def ozero_renamer():
    r = "O" + "".join(choice("0O") for i in range(19))
    while r in ozero_cache:
        r = "O" + "".join(choice("0O") for i in range(19))
    return r


lone_cache = set()


def lone_renamer():
    r = "l" + "".join(choice("1l") for i in range(19))
    while r in lone_cache:
        r = "l" + "".join(choice("1l") for i in range(19))
    return r


is_magic = lambda x: len(x) > 4 and all((x[i] == "_") for i in [0, 1, -1, -2])


class RenameVariables:
    """
    Variable renamer, renames variables, functions, classes and decorators
    Currently doesn't support these:
    1. Names imported from modules
    2. Keyword arguments
    3. Renaming variables differently in different scopes
    4. eval() and exec() may not work correctly
    :param rename_variables: Pass this argument to enable
    :param renamer: The renamer to use, 'hex' for hex renamer, 'char' for random letter renamer, 'o0' for O and 0
    renamer, 'underscore' for underscore renamer and 'l1' for l and 1 renamer, hex renamer is used if not passed.
    :param obfuscation_list: Optional, a dictionary for names that must be renamed to exactly the corresponding value in dictionary. Usually used for names that are not
    supposed to be renamed, or names from other obfuscated files.
    :param exports: Names that are renamed but needed to be exported, it is recommended to use this with string obfuscations
    """

    priority = 4

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        global underscore_cache, hex_cache, char_cache, ozero_cache, lone_cache
        builtins_val = [  # A list of builtin functions
            "ArithmeticError",
            "AssertionError",
            "AttributeError",
            "BaseException",
            "BlockingIOError",
            "BrokenPipeError",
            "BufferError",
            "BytesWarning",
            "ChildProcessError",
            "ConnectionAbortedError",
            "ConnectionError",
            "ConnectionRefusedError",
            "ConnectionResetError",
            "DeprecationWarning",
            "EOFError",
            "Ellipsis",
            "EnvironmentError",
            "Exception",
            "False",
            "FileExistsError",
            "FileNotFoundError",
            "FloatingPointError",
            "FutureWarning",
            "GeneratorExit",
            "IOError",
            "ImportError",
            "ImportWarning",
            "IndentationError",
            "IndexError",
            "InterruptedError",
            "IsADirectoryError",
            "KeyError",
            "KeyboardInterrupt",
            "LookupError",
            "MemoryError",
            "ModuleNotFoundError",
            "NameError",
            "None",
            "NotADirectoryError",
            "NotImplemented",
            "NotImplementedError",
            "OSError",
            "OverflowError",
            "PendingDeprecationWarning",
            "PermissionError",
            "ProcessLookupError",
            "RecursionError",
            "ReferenceError",
            "ResourceWarning",
            "RuntimeError",
            "RuntimeWarning",
            "StopAsyncIteration",
            "StopIteration",
            "SyntaxError",
            "SyntaxWarning",
            "SystemError",
            "SystemExit",
            "TabError",
            "TimeoutError",
            "True",
            "TypeError",
            "UnboundLocalError",
            "UnicodeDecodeError",
            "UnicodeEncodeError",
            "UnicodeError",
            "UnicodeTranslateError",
            "UnicodeWarning",
            "UserWarning",
            "ValueError",
            "Warning",
            "WindowsError",
            "ZeroDivisionError",
            "_",
            "__build_class__",
            "__debug__",
            "__doc__",
            "__import__",
            "__loader__",
            "__name__",
            "__package__",
            "__spec__",
            "abs",
            "all",
            "any",
            "ascii",
            "bin",
            "bool",
            "breakpoint",
            "bytearray",
            "bytes",
            "callable",
            "chr",
            "classmethod",
            "compile",
            "complex",
            "copyright",
            "credits",
            "delattr",
            "dict",
            "dir",
            "divmod",
            "enumerate",
            "eval",
            "exec",
            "exit",
            "filter",
            "float",
            "format",
            "frozenset",
            "getattr",
            "globals",
            "hasattr",
            "hash",
            "help",
            "hex",
            "id",
            "input",
            "int",
            "isinstance",
            "issubclass",
            "iter",
            "len",
            "license",
            "list",
            "locals",
            "map",
            "max",
            "memoryview",
            "min",
            "next",
            "object",
            "oct",
            "open",
            "ord",
            "pow",
            "print",
            "property",
            "quit",
            "range",
            "repr",
            "reversed",
            "round",
            "set",
            "setattr",
            "slice",
            "sorted",
            "staticmethod",
            "str",
            "sum",
            "super",
            "tuple",
            "type",
            "vars",
            "zip",
            "__builtins__",
        ]
        underscore_cache, ozero_cache, lone_cache, char_cache, hex_cache = (
            1,
            set(),
            set(),
            set(),
            set(),
        )
        if "rename_variables" not in self.options:
            return code
        if "renamer" not in self.options:
            renamer = hex_renamer
        else:
            renamer = {
                "hex": hex_renamer,
                "char": character_renamer,
                "o0": ozero_renamer,
                "underscore": underscore_renamer,
                "l1": lone_renamer,
            }[self.options["renamer"]]
        scanned_names = {}
        obfulist = {}
        if "obfuscation_list" in self.options:
            scanned_names = self.options["obfuscation_list"]
            obfulist = self.options["obfuscation_list"]
        exports = set()
        if "exports" in self.options:
            exports = set(self.options["exports"])
        result = ""
        prev_toktype, prev_tokval = "", ""
        string_removed = ""
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if not is_token_subtype(toktype, String):
                string_removed += tokval
        modules = set()
        for i in string_removed.split("\n"):
            j = i.split()
            if len(j) >= 2:
                if j[0] in ["from", "import"]:
                    modules |= set(map(lambda x: x.strip(), j[1].split(",")))
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if (
                (
                    toktype == Token.Name
                    or toktype == Token.Name.Function
                    or toktype == Token.Name.Class
                    or toktype == Token.Name.Decorator
                    or toktype == Token.Name.Builtin.Pseudo
                )
                and (tokval not in scanned_names and (tokval not in builtins_val))
                and (
                    not (
                        prev_tokval == "."
                        and prev_toktype == Token.Operator
                        and (tokval not in scanned_names)
                    )
                )
                and (tokval not in modules)
            ):
                scanned_names[tokval] = renamer()
                if tokval[0] == "@":  # For decorators
                    scanned_names[tokval] = "@" + scanned_names[tokval]
            prev_toktype = toktype
            prev_tokval = tokval
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if (
                (
                    toktype == Token.Name
                    or toktype == Token.Name.Function
                    or toktype == Token.Name.Class
                    or toktype == Token.Name.Decorator
                    or toktype == Token.Name.Builtin.Pseudo
                )
                and (tokval in scanned_names and (tokval not in builtins_val))
                and (not is_magic(tokval))
            ):
                result += scanned_names[tokval]
            else:
                result += tokval
        for i in exports:
            if (i in scanned_names) and (i not in obfulist):
                result += (
                    "\n"
                    + '(setattr(__builtins__,"{}",{}) if not isinstance(__builtins__,dict) else __builtins__.__setitem__("{}",{}))'.format(
                        i, scanned_names[i], i, scanned_names[i]
                    )
                )
        return result


__all__ = ["RenameVariables"]



import random

import pygments
from pygments.lexers import PythonLexer
from pygments.token import *
from random import *

randbytes = lambda x: bytes(randint(0, 255) for i in range(x))


class StringOBF:
    """
    String obfuscation: merge all strings into one large bytes and replace with slices
    :param string_obf: Pass this argument to enable
    :param random_strings: Add random strings in the strings
    :param encrypt: Encryption method: 'base64' for base64, 'base64-lzma' for base64+lzma
    """

    priority = 5

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        if "string_obf" not in self.options:
            return code
        result = ""
        open_bracket = 0
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if is_token_subtype(toktype, Token.String) and tokval == "\n":
                result += "\\n"
                continue
            else:
                open_bracket += (
                    tokval.count("(")
                    + tokval.count("[")
                    + tokval.count("{")
                    - tokval.count("}")
                    - tokval.count("]")
                    - tokval.count(")")
                )
                if tokval == "\n" and open_bracket:
                    continue
            result += tokval
        code = result
        code += "\n"
        affix = ""
        string = ""
        strings = []
        pos = 0
        startpos = -1
        fstring_bracket = False
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if toktype == String.Interpol:
                fstring_bracket = not fstring_bracket
                string += tokval
            elif fstring_bracket:
                string += tokval
            elif toktype == String.Affix:
                affix = tokval
                if startpos == -1:
                    startpos = pos
            elif is_token_subtype(toktype, String):
                if startpos == -1:
                    startpos = pos
                string += tokval
            else:
                if startpos != -1:
                    strings.append((affix, string, startpos, pos))
                    affix, string, startpos = "", "", -1
            pos += len(tokval)
        replacements = []
        stringtable = b""
        for af, s, st, ed in strings:
            result = "$"
            raw_af = af
            notbytes = False
            if "b" not in af:
                result += ".decode()"
                notbytes = True
            if "f" in af:
                result = "eval(chr(102)+repr(" + result + "))"
                raw_af = raw_af.replace("f", "")
            if notbytes:
                s2 = eval(raw_af + s).encode()
            else:
                s2 = eval(raw_af + s)
            replacements.append((s2, result, st, ed))
        slices = []
        for s, fmt, st, ed in replacements:
            if "random_strings" in self.options:
                stringtable += randbytes(randint(1, 20))
            slices.append((len(stringtable), len(stringtable) + len(s)))
            stringtable += s
        if "encrypt" not in self.options:
            res = "_s={}\n".format(repr(stringtable))
        elif self.options["encrypt"] == "base64":
            import base64

            res = "import base64\n_s=base64.b64decode({})\n".format(
                repr(base64.b64encode(stringtable))
            )
        elif self.options["encrypt"] == "base64-lzma":
            import base64, lzma

            res = (
                "import base64,lzma\n_s=lzma.decompress(base64.b64decode({}))\n".format(
                    repr(base64.b64encode(lzma.compress(stringtable)))
                )
            )
        p, P = 0, 0
        while p < len(code):
            if P < len(replacements) and p == replacements[P][2]:
                res += replacements[P][1].replace(
                    "$", "_s[{}:{}]".format(slices[P][0], slices[P][1])
                )
                p = replacements[P][3]
                P += 1
            else:
                res += code[p]
                p += 1
        return res


__all__ = ["StringOBF"]



"""
Obfuscation methods
"""
from .execexp import ExecExp
from .remove_comments import RemoveComments
from .rename_variables import RenameVariables
from .add_code import AddCode
from .error_free import ErrorFree
from .stringobf import StringOBF
from .magic import Magic

obfuscation_methods = [
    ExecExp,
    RemoveComments,
    RenameVariables,
    AddCode,
    ErrorFree,
    StringOBF,
    Magic,
]



from obfsimp import Obfuscator
import sys


def yesno(prompt):
    inp = input(prompt + " (y/n) ")
    while inp not in list("yYnN"):
        inp = input(prompt + " (y/n) ")
    return inp.lower() == "y"


def _cli():
    print("Welcome to the obfsimp Python obfuscator!")
    fn = input("Input file name you want to obfuscate: ")
    options = {}
    try:
        with open(fn, "r", encoding="utf-8") as f:
            code = f.read()
    except BaseException as e:
        print("Cannot read file, error is: " + str(e))
        sys.exit()
    if yesno("Do you want to remove comments and docstrings?"):
        options["remove_comments"] = True
    if yesno("Do you want to wrap the program in an exec expression?"):
        options["exec"] = True
        enc = input(
            "What encryption do you want to use? ((n)o/(b)ase64/base64-(l)zma/(e)scape) "
        )
        while enc.lower() not in [
            "n",
            "b",
            "l",
            "e",
            "no",
            "base64",
            "base64-lzma",
            "escape",
        ]:
            enc = input(
                "What encryption do you want to use? ((n)o/(b)ase64/base64-(l)zma/(e)scape) "
            )
        enc = enc.lower()
        if enc != "n" and enc != "no":
            if enc == "b":
                enc = "base64"
            if enc == "l":
                enc = "base64-lzma"
            if enc == "e":
                enc = "escape"
            options["exec_crypt"] = enc
    if yesno("Do you want the program to ignore errors?"):
        options["error_free"] = True
    if yesno("Do you want to obfuscate variable names?"):
        options["rename_variables"] = True
        renamer = input(
            "What renamer do you want to use? ((h)ex/(o)0/(l)1/(u)nderscore/(c)har) default is hex "
        )
        if renamer.lower() not in [
            "h",
            "o",
            "l",
            "u",
            "c",
            "hex",
            "o0",
            "l1",
            "underscore",
            "char",
        ]:
            renamer = "hex"
        renamer = renamer.lower()
        if renamer == "h":
            renamer = "hex"
        if renamer == "l":
            renamer = "l1"
        if renamer == "o":
            renamer = "o0"
        if renamer == "u":
            renamer = "underscore"
        if renamer == "c":
            renamer = "char"
        options["renamer"] = renamer
        inp = input(
            "Do you want to export some names? separate them with spaces (If you don't want just input nothing) "
        ).split()
        options["exports"] = inp
    if yesno("Do you want to obfuscate strings?"):
        options["string_obf"] = True
        if yesno("Do you want to add random strings?"):
            options["random_strings"] = True
        enc = input(
            "What encryption do you want to use? ((n)o/(b)ase64/base64-(l)zma) "
        )
        while enc.lower() not in ["n", "b", "l", "no", "base64", "base64-lzma"]:
            enc = input(
                "What encryption do you want to use? ((n)o/(b)ase64/base64-(l)zma) "
            )
        enc = enc.lower()
        if enc != "n" and enc != "no":
            if enc == "b":
                enc = "base64"
            if enc == "l":
                enc = "base64-lzma"
            if enc == "e":
                enc = "escape"
            options["encrypt"] = enc
    if yesno("Do you want to add junk code?"):
        options["add_code"] = True
        num = input("How much junk code do you want to add? default is 10 ")
        if not num.isdigit():
            num = 10
        else:
            num = int(num)
        options["code_amount"] = num
    if yesno(
        "Do you want to convert code to code that only uses magic functions? (Only suitable for small programs)"
    ):
        options["magic"] = True
    print("Start obfuscating...")
    o = Obfuscator(**options)
    obfuscated = o(code)
    print("Done!")
    fn = input("Output file name: ")
    try:
        with open(fn, "w", encoding="utf-8") as f:
            f.write(obfuscated)
    except BaseException as e:
        print("Cannot write file, error is: " + str(e))
        sys.exit()
    print("That's it! obfuscated code has been successfully written to " + fn)



import pygments
from pygments.lexers import PythonLexer
from pygments.token import *


class RemoveComments:
    """
    Comment remover, removes comments and docstrings
    :param remove_comments: Pass this argument to enable
    """

    priority = 0xFFFFFFFFFFFFFFFF  # Priority must be the highest

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        if "remove_comments" not in self.options:
            return code
        result = ""
        for toktype, tokval in pygments.lex(code, PythonLexer()):
            if not is_token_subtype(toktype, Token.Comment) and not is_token_subtype(
                toktype, String.Doc
            ):
                if is_token_subtype(toktype, Token.String) and tokval == "\n":
                    result += "\\n"
                    continue
                result += tokval
        return result


__all__ = ["RemoveComments"]



"""
Obfsimp is a simple Python 3 obfuscator that is mainly used for entertainment.

Needs to be used in Python 3, and better use the Python version the same as the version you want to execute the
obfuscated program with to obfuscate.
"""
from .methods import obfuscation_methods


class Obfuscator:
    """
    Class for obfuscating Python code
    """

    def __init__(self, **options):
        """
        Initialize object
        :param options: Options for obfuscation, it is used as flags by methods
        """
        self.options = options
        self.methods = []
        for i in obfuscation_methods:
            self.methods.append(i(**self.options))
        self.methods.sort(key=lambda x: x.priority, reverse=True)

    def __call__(self, code, checksyntax=True):
        """
        Obfuscate
        :param code: Python code
        :param checksyntax: Check if the syntax is valid before obfuscation
        :return: obfuscated code
        """
        if checksyntax:
            try:
                compile(code, "", "exec")
            except SyntaxError:
                raise SyntaxError(
                    "Your code does not have a valid syntax, to silence this error, use checksyntax=False"
                )
        for i in self.methods:
            code = i(code)
        return code



from setuptools import setup

docs = """
Simple Python 3 obfuscator

Support many techniques, such as exec expression, variable renaming, etc.

To use it, simply type "obfsimp" in terminal

There are many example programs and obfuscated programs at the examples folder
"""
setup(
    name="obfsimp",
    version="1.0.0",
    description="Simple Python 3 obfuscator",
    long_description=docs,
    packages=["obfsimp", "obfsimp.methods", "obfsimp.examples"],
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Operating System :: OS Independent",
    ],
    keywords=["obfuscator"],
    entry_points={"console_scripts": ["obfsimp=obfsimp.cli:_cli"]},
    install_requires=["pygments"],
    python_requires=">=3",
)



class ErrorFree:
    """
    Disable errors
    :param error_free: Pass this argument to enable
    """

    priority = 0

    def __init__(self, **options):
        self.options = options

    def __call__(self, code):
        if "error_free" not in self.options:
            return code
        s = code.split("\n")
        t = ["try:"]
        for i in s:
            t.append("    " + i)
        t.append("except:\n    pass")
        return "\n".join(t)


__all__ = ["ErrorFree"]
